# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import importlib
import inspect
import json
import re
import sys

from django.conf import settings
from django.http import HttpResponse

from django.core.serializers.json import Serializer, DjangoJSONEncoder
from django.db import models

regexes = {
    'bootspeed': re.compile(
        ur"^bootspeed-(milestone|backfill|raring|saucy)-(desktop)-([^-]*)-(.*)"
    ),
    'power': re.compile(
        ur"^power-(milestone|backfill|raring|saucy)-(desktop)-([^-]*)-(.*)"
    ),
    'idlepower': re.compile(
        ur"^poweridle-(milestone|backfill|raring|saucy)-(desktop)-([^-]*)-(.*)"
    ),
    'upgrade': re.compile(
        ur".*-upgrade-.*"
    ),
    'smoke': re.compile(
        ur"^(precise|quantal)-(desktop|server|alternate)"
    ),
    'utah-smoke': re.compile(
        ur"^(raring|saucy)-(desktop|server)-.*-smoke-.*"
    ),
    'utah-smoke-data': re.compile(
        ur"^(raring|saucy)-(desktop|server)-([^-]*)-smoke-(.*)"
    ),
    'smoke-ec2': re.compile(
        ur"(.*)-(.*)-ec2.*"
    ),
    'smoke-ec2-daily': re.compile(
        ur".*-daily$"
    ),
    'smoke-ec2-daily-data': re.compile(
        ur"^(.*)-(.*).*"
    ),
    'smoke-data': re.compile(
        ur"^(precise|quantal)-(.*)-([^_]*)_(.*)"
    ),
    'smoke-buildid': re.compile(
        r"^[0-9]{8}(\.[0-9]+)?$"
    ),
    'smoke-build_desc': re.compile(
        ur"[^,\s]+"
    ),
    'kernel_sru': re.compile(
        ur"^(sru_kernel)(|_backport)-" +
        "(lucid|precise|quantal|raring|saucy)(|_lts_hwe)-(.*)-(.*)-(.*).*"
    ),
    'kernel_sru-version': re.compile(
        ur"^([^,\s]+)(\s+|,)"
    ),
    'lpbug': re.compile(
        ur"(?i)^LP[:\s#]*([0-9]+)$"
    ),
    'lpbug-job': re.compile(
        ur"JOB-LP:#[0-9]+$"
    ),
    'buildid': re.compile(
        r"^(|[0-9]+:)[0-9]{8}(\.[0-9]+)?(|:[0-9]{8}(\.[0-9]+)?)?$"
    ),
    'build_desc': re.compile(
        ur"[^,]+"
    ),
    'build-number': re.compile(
        ur"^.*\((.+)\)$"
    ),
    'console-build-number': re.compile(
        ur'.*Media Info:.*\(([0-9.]+)\)'
    ),
    'artifact-ignore': re.compile(
        ur"^tests\/"
    ),
    'skip_build': re.compile(
        ur"dash_ignore"
    ),
}


def JSONResponse(data):
    """
    Convenience method.
    """
    return HttpResponse(data, content_type='application/json')

# From: http://stackoverflow.com/q/5453237/1535294


class MySerializer(Serializer):
    def __init__(self, use_extra_fields=False):
        super(MySerializer, self).__init__()

        self.use_extra_fields = use_extra_fields

    def end_object(self, obj):
        self._current['id'] = obj._get_pk_val()

        # Add any property fields listed to the object representation
        # XXX: This slows things down considerably, look for a better way.
        if self.use_extra_fields and hasattr(obj, 'extra_fields'):
            attrs = getattr(obj, 'extra_fields')

            for attr in attrs:
                if hasattr(obj, attr):
                    self._current[attr] = getattr(obj, attr)

        json.dump(self._current, self.stream, cls=DjangoJSONEncoder, **self.options)


def my_models(name):
    mymodels = []
    for x in inspect.getmembers(sys.modules[name], inspect.isclass):
        klass = x[1]
        if issubclass(klass, models.Model) and klass.__module__ == name:

            # skip abstract base classes.
            if hasattr(klass, '_meta') and klass._meta.abstract:
                continue
            mymodels.append(klass)

    return mymodels


def get_class_map():

    klass_map = dict()

    for app in settings.LOCAL_APPS:
        mod = importlib.import_module("{}.models".format(app))
        klass_map[app] = my_models(mod.__name__)

    return klass_map


def truncate(value, limit=55):
    """
    Based on http://djangosnippets.org/snippets/1259/
    """
    # Return the string itself if length is smaller or equal to the limit
    if len(value) <= limit:
        return value

    # Cut the string
    value = value[:limit]

    # Join the words and return
    return value + '...'


def wrap_in_oops_wsgi(wsgi_handler):
    import oops_dictconfig
    from qa_dashboard.settings import oops_config
    from oops_wsgi import install_hooks, make_app
    cfg = oops_dictconfig.config_from_dict(oops_config)
    cfg.template['reporter'] = 'ci-dashboard'
    install_hooks(cfg)
    return make_app(wsgi_handler, cfg, oops_on_status=['500'])
