# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models

from common.models import (
    DashboardBaseModel,
    Bug,
    JenkinsBuild,
)


class SmokeImage(DashboardBaseModel):
    class Meta:
        db_table = 'smoke_images'

    extra_fields = [
        'summary',
    ]


    build_number = models.CharField(max_length=200)
    build_id = models.CharField(max_length=200, default='')
    rootfs_id = models.CharField(max_length=200, default='')
    device_specific_id = models.CharField(max_length=200, default='')
    customization_id = models.CharField(max_length=200, default='')

    release = models.CharField(max_length=200)
    flavor = models.CharField(max_length=200)
    arch = models.CharField(max_length=200)
    variant = models.CharField(max_length=200)

    def __unicode__(self):
        return "{} {} {} {}".format(
            self.release,
            self.variant,
            self.arch,
            self.build_number,
        )

    @property
    def summary(self):
        data = dict(
            pass_count=0,
            fail_count=0,
            error_count=0,
            total_count=0,
            pass_rate=0,
        )

        results = self.smokeresult_set.filter(
            publish=True,
        ).values(
            'id',
            'image__build_number',
            'image__variant',
            'image__arch',
            'image__id',
            'image__release',
            'pass_count',
            'fail_count',
            'error_count',
            'total_count',
            'name',
            'ran_at',
            'jenkins_build__job__url',
            'jenkins_build__build_number',
        )
        crash_count = self.smokeresult_set.filter(
            publish=True,
            jenkins_build__artifact__name__contains='crash',
        ).values(
            'id',
        )

        bugs = SmokeResultBug.objects.filter(
            result__image=self
        ).values(
            'result__id',
            'bug__bug_no',
            'bug__title',
            'bug__status',
            'bug__assignee',
            'bug__importance',
            'bug__project',
        )

        bug_data = {}
        for bug in bugs:
            if bug['bug__bug_no'] not in bug_data:
                bug_info = dict(
                    bug_number=bug['bug__bug_no'],
                    title=bug['bug__title'],
                    assignee=bug['bug__assignee'],
                    status=bug['bug__status'],
                    importance=bug['bug__importance'],
                    project=bug['bug__project'],
                )
                bug_data[bug['bug__bug_no']] = bug_info

        # we need to give a non-queryset struct to the caller since we are
        # calculating passrates and bugs for this
        results_copy = []
        passrates = []
        for r in results:
            r['bugs'] = []
            r['crash_count'] = 0
            data['pass_count'] += r['pass_count']
            data['fail_count'] += r['fail_count']
            data['error_count'] += r['error_count']
            data['total_count'] += r['total_count']
            r['pass_rate'] = 0
            if r['total_count']:
                r['pass_rate'] = float(r['pass_count']) / r['total_count']
            r['pass_rate_pct'] = 100 * r['pass_rate']
            passrates.append(r['pass_rate'])

            for b in bugs:
                if b['result__id'] == r['id']:
                    r['bugs'].append(b['bug__bug_no'])

            for crashes in crash_count:
                if crashes['id'] == r['id']:
                    r['crash_count'] += 1

            results_copy.append(r)

        if len(passrates) == 0:
            data['pass_rate'] = 0
        else:
            data['pass_rate'] = sum(passrates) / len(passrates)
        data['pass_rate_pct'] = data['pass_rate'] * 100
        data['results'] = results_copy

        return data, bug_data


class SmokeResult(DashboardBaseModel):
    class Meta:
        db_table = 'smoke_results'
    image = models.ForeignKey(SmokeImage)
    jenkins_build = models.ForeignKey(JenkinsBuild)
    name = models.CharField(max_length=255)
    fail_count = models.IntegerField()
    error_count = models.IntegerField(null=False, default=0)
    pass_count = models.IntegerField()
    total_count = models.IntegerField()
    ran_at = models.DateTimeField('date run')
    pass_rate = models.FloatField(default=0.0)

    def __unicode__(self):
        return "{} - {}".format(
            self.name,
            self.ran_at,
        )


class SmokeResultBug(DashboardBaseModel):
    class Meta:
        db_table = "smoke_result_bugs"

    bug = models.ForeignKey(Bug)
    result = models.ForeignKey(SmokeResult)


class SmokeTestResult(DashboardBaseModel):
    class Meta:
        db_table = 'smoke_tests'
    result = models.ForeignKey(SmokeResult)
    command_type = models.CharField(max_length=4096)
    command = models.CharField(max_length=4096)
    testcase = models.CharField(max_length=4096)
    testsuite = models.CharField(max_length=4096)
    returncode = models.IntegerField()
    stderr = models.TextField(null=True)
    stdout = models.TextField(null=True)
