# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import mock
import urllib2

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client

from smokeng.models import (
    SmokeImage,
    SmokeResult,
    SmokeResultBug,
    SmokeTestResult,
)

from smokeng.management.commands.jenkins_pull_smokeng import Command

from common.models import (
    Bug,
    JenkinsBuild,
    JenkinsJob,
)


def _setUp(self):
    """ Basic setup for each test. """

    self.jenkins_job = JenkinsJob.objects.create(
        name="blah-blah-blah",
        url="http://blah.com",
    )
    self.assert_(self.jenkins_job)

    self.jenkins_build = JenkinsBuild.objects.create(
        job=self.jenkins_job,
        build_number=1,
        ran_at=datetime.datetime.now(),
    )
    self.assert_(self.jenkins_build)

    self.image = SmokeImage.objects.create(
        release="saucy",
        arch="amd64",
        variant="desktop",
        build_number="20130401",
    )
    self.assert_(self.image)

    self.result = SmokeResult.objects.create(
        image=self.image,
        jenkins_build=self.jenkins_build,
        name="oem",
        fail_count=1,
        error_count=1,
        pass_count=1,
        total_count=3,
        ran_at=datetime.datetime.now(),
    )
    self.assert_(self.result)

    self.bug = Bug.objects.create(
        bug_no=1,
        status='New',
        importance='Low',
        assignee='Chris Johnston',
    )
    self.assert_(self.bug)

    self.result_bug = SmokeResultBug.objects.create(
        bug=self.bug,
        result=self.result,
    )
    self.assert_(self.result_bug)

    self.testcaseresult = SmokeTestResult.objects.create(
        result=self.result,
        command_type="testcase_test",
        command="./test.py",
        testcase="test-testcase",
        testsuite="touch",
        returncode="0",
    )


class TestSmokeURLS(TestCase):

    def setUp(self):
        self.c = Client()

        _setUp(self)

    def _test_url(self, url):
        response = self.c.get(url)

        self.assertEqual(response.status_code, 200)

    def test_release(self):

        url = reverse(
            'smokeng_overview',
            args=(
                self.image.release,
            ),
        )

        self._test_url(url)

    def test_release_bug_sorting(self):

        url = reverse(
            'smokeng_overview',
            args=(
                self.image.release,
            ),
        )

        self._test_url("{}?sort=bugs".format(url))

    def test_image_overview_url(self):

        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )
        self._test_url(url)

    def test_image_overview_touch_ro_url(self):
        self.image.build_number = '56:20130920:20130920'
        self.image.save()
        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )
        self._test_url(url)

    def test_image_overview_unknown_build_number_url(self):
        self.image.build_number = '20130920 ?'
        self.image.save()
        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )

        # URL quote the url due to the '?'
        url = urllib2.quote(url)
        self._test_url(url)

    def test_image_bug_sorting(self):

        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )

        self._test_url("{}?sort=bugs".format(url))

    def test_image_test_results_url(self):
        url = reverse(
            'smokeng_image_test_results',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
            ),
        )
        response = self._test_url(url)
        self.assertTemplateUsed(response, 'smokeng/test_results.html')

    def test_image_test_results_touch_ro_url(self):
        self.image.build_number = '56:20130920:20130920'
        self.image.save()
        url = reverse(
            'smokeng_image_test_results',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
            ),
        )
        self._test_url(url)

    def test_image_test_results_unknown_build_number_url(self):
        self.image.build_number = '20130920 ?'
        self.image.save()
        url = reverse(
            'smokeng_image_test_results',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
            ),
        )

        # URL quote the url due to the '?'
        url = urllib2.quote(url)
        self._test_url(url)

    def test_test_result_detail_url(self):
        url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )
        response = self._test_url(url)
        self.assertTemplateUsed(response, 'smokeng/test_result_detail.html')

    def test_image_test_result_detail_touch_ro_url(self):
        self.image.build_number = '56:20130920:20130920'
        self.image.save()
        url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )
        self._test_url(url)

    def test_image_test_result_detail_unknown_build_number_url(self):
        self.image.build_number = '20130920 ?'
        self.image.save()
        url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )

        # URL quote the url due to the '?'
        url = urllib2.quote(url)
        self._test_url(url)


class TestSmoke(TestCase):
    """ Tests for the smoke models. """

    def setUp(self):
        _setUp(self)

    def test_image_aggregations_pass(self):
        """ Test aggregated pass values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['pass_count'], 1)

    def test_image_aggregations_fail(self):
        """ Test aggregated fail values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['fail_count'], 1)

    def test_image_aggregations_error(self):
        """ Test aggregated error values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['error_count'], 1)

    def test_image_aggregations_total(self):
        """ Test aggregated total values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['total_count'], 3)

    def test_image_aggregations_pass_rate(self):
        """ Test aggregated pass_rate values for an image instance. """
        summary, bug_data = self.image.summary

        # cast to int to make comparison more reliable
        self.assertEqual(int(summary['pass_rate'] * 100), 33)

    def test_result_bugs(self):
        """ Test bug list for a result instance. """

        self.assertEqual(self.result.smokeresultbug_set.count(), 1)


class TestSmokeMethods(TestCase):

    def setUp(self):
        self.command = Command()
        self.build = dict(
            url="http://blah",
            artifacts=[
                dict(
                    fileName="media_info",
                ),
            ],
        )

    def test_get_build_number_none(self):
        build = {}

        self.assertIsNone(self.command.get_build_number(build))

    @mock.patch('urllib2.Request')
    @mock.patch('urllib2.urlopen')
    def test_get_build_number_media_info(self, urlopen, request):
        build_number = '20111012'
        url_resp = 'Ubuntu 11.10 "Oneiric Ocelot" - Release amd64 ({})'.format(
            build_number,
        )
        contents = mock.Mock()
        contents.read.return_value = url_resp
        urlopen.return_value = contents

        self.assertEqual(
            self.command.get_build_number(self.build),
            build_number,
        )

    def test__has_media_info_true(self):

        self.assertTrue(self.command._has_media_info(self.build))

    def test__has_media_info_false(self):

        self.build = {}

        self.assertFalse(self.command._has_media_info(self.build))
