# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from django.db import models
from django.shortcuts import (
    render_to_response,
    get_object_or_404,
)
from django_tables2 import RequestConfig
from django.template import RequestContext
from django.views.decorators.http import require_GET

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)

from common.models import Artifact
from common.views import index
from common.templatetags.dashboard_extras import decimal_to_percent

from smokeng.config import primary_targets
from smokeng.models import (
    SmokeImage,
    SmokeResult,
    SmokeResultBug,
    SmokeTestResult,
)
from smokeng.tables import (
    SmokeImageTable,
    SmokeResultTable,
    SmokeTestResultTable,
)


def _get_releases():
    return [
        image['release'] for image in SmokeImage.objects.values(
            'release',
        ).order_by('-release').distinct().exclude(publish=False)
    ]


def _get_variants(release):
    return [
        image['variant'] for image in SmokeImage.objects.filter(
            release=release,
        ).values(
            'variant',
        ).order_by('-variant').distinct().exclude(publish=False)
    ]


def get_totals(results, release):
    totals = {}
    passrates = {}

    bugs = _get_bugs(release)
    crashes = _get_crashes()
    bug_data = {}
    for result in results:
        _append_totals(totals, result, crashes)
        if result['total_count'] == 0:
            passrate = 0
        else:
            passrate = float(result['pass_count']) / result['total_count']

        passrates.setdefault(result['image__id'], []).append(passrate)

    for image, passrate in passrates.iteritems():
        totals[image]['pass_rate'] = \
            sum(passrates[image]) / len(passrates[image])
        totals[image]['pass_rate_pct'] = \
            totals[image]['pass_rate'] * 100

    for bug in bugs.iterator():
        if bug.result.image.id in totals:
            if bug.bug.bug_no not in totals[bug.result.image.id]['bugs']:
                totals[bug.result.image.id]['bugs'].append(bug.bug.bug_no)
        if bug.bug.bug_no not in bug_data:
            bug_info = dict(
                bug_number=bug.bug.bug_no,
                title=bug.bug.title,
                assignee=bug.bug.assignee,
                status=bug.bug.status,
                importance=bug.bug.importance,
                project=bug.bug.project,
            )
            bug_data[bug.bug.bug_no] = bug_info
    return totals, bug_data


def _get_crashes():
    crashes = SmokeResult.objects.filter(
        publish=True,
        jenkins_build__artifact__name__contains='crash',
    ).values(
        'image__build_number',
        'image__arch',
        'image__variant',
    )
    return crashes


def _get_bugs(release):

    bugs = SmokeResultBug.objects.filter(
        result__image__release=release,
    ).select_related().order_by('bug')
    return bugs


def _append_totals(totals, result, crashes):
    key = result['image__id']

    if key not in totals:
        totals[key] = dict(
            build_id=result['image__build_id'],
            rootfs_id=result['image__rootfs_id'],
            device_specific_id=result['image__device_specific_id'],
            customization_id=result['image__customization_id'],
            build_number=result['image__build_number'],
            release=result['image__release'],
            total_count=0,
            pass_count=0,
            fail_count=0,
            error_count=0,
            bugs=[],
            name=result['name'],
            crash_count=0,
        )

        for crash in crashes:
            build_number = crash['image__build_number']
            if (
                build_number == result['image__build_number'] and
                crash['image__arch'] == result['image__arch'] and
                crash['image__variant'] == result['image__variant']
            ):
                totals[key]['crash_count'] += 1

        if 'image__variant' in result:
            totals[key]['variant'] = result['image__variant']

        if 'image__id' in result:
            totals[key]['image_id'] = result['image__id']

        if 'image__arch' in result:
            totals[key]['arch'] = result['image__arch']

        if 'ran_at' in result:
            totals[key]['ran_at'] = result['ran_at']

    totals[key]['total_count'] += result['total_count']
    totals[key]['pass_count'] += result['pass_count']
    totals[key]['error_count'] += result['error_count']
    totals[key]['fail_count'] += result['fail_count']


@BreadCrumb("{release} image testing", parent=index, needs=['release'])
@require_GET
def overview(request, release=None, variant=None):

    if release is None:
        releases = _get_releases()
        release = releases[0]

    variants = _get_variants(release)

    results = SmokeResult.objects.filter(
        publish=True,
        image__release=release,
    )
    targets = ''
    if variant is not None:
        results = results.filter(
            image__variant=variant,
        )

        primary_tgts = primary_targets(variant)
        if request.GET.get('show_all') != '1' and primary_tgts:
            targets = 'primary'
            results = results.filter(image__arch__in=primary_tgts)

    results = results.values(
        'image__build_number',
        'image__variant',
        'image__arch',
        'image__id',
        'image__build_id',
        'image__rootfs_id',
        'image__device_specific_id',
        'image__customization_id',
        'image__release',
        'name',
        'ran_at',
        'jenkins_build',
    ).annotate(
        pass_count=models.Sum('pass_count'),
        fail_count=models.Sum('fail_count'),
        error_count=models.Sum('error_count'),
        total_count=models.Sum('total_count'),
    ).values(
        'image__build_id',
        'image__rootfs_id',
        'image__device_specific_id',
        'image__customization_id',
        'image__build_number',
        'image__variant',
        'image__arch',
        'image__id',
        'image__release',
        'name',
        'ran_at',
        'jenkins_build',
        'pass_count',
        'fail_count',
        'error_count',
        'total_count',
    )

    totals, bug_data = get_totals(results, release)
    table = SmokeImageTable(totals.values())
    RequestConfig(request, paginate=None).configure(table)

    data = {
        'table': table,
        'bug_data': bug_data,
        'release': release,
        'releases': _get_releases(),
        'variants': variants,
        'current': variant,
        'targets': targets,
        'active': release,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            overview,
            release=release,
        )
    }

    return render_to_response(
        'smokeng/overview.html',
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Image {build_number}",
    parent=overview,
    needs=['release', 'variant', 'arch', 'build_number', 'image_id'],
)
@require_GET
def image_overview(request, release, variant, arch, build_number, image_id):

    image = get_object_or_404(SmokeImage, id=image_id)
    summary, bug_data = image.summary

    table = SmokeResultTable(summary['results'])
    RequestConfig(request, paginate=None).configure(table)

    data = {
        'image': image,
        'bug_data': bug_data,
        'summary': summary,
        'build_number': image.build_number,
        'release': release,
        'table': table,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_overview,
            release=release,
            variant=variant,
            arch=arch,
            image_id=image_id,
            build_number=image.build_number,
        ),
    }
    return render_to_response(
        'smokeng/image_overview.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "{name}",
    parent=image_overview,
    needs=['release', 'variant', 'arch', 'build_number', 'image_id', 'name'],
)
@require_GET
def image_test_results(
    request,
    release,
    variant,
    arch,
    build_number,
    image_id,
    name,
):

    image = get_object_or_404(SmokeImage, id=image_id)
    result = get_object_or_404(
        SmokeResult,
        image__id=image_id,
        name=name,
        publish=True,
    )
    tests = SmokeTestResult.objects.filter(
        publish=True,
        result=result,
    )
    artifacts = Artifact.objects.filter(
        jenkins_build=result.jenkins_build,
    ).order_by('name')

    jenkins_url = (
        result.jenkins_build.job.url +
        result.jenkins_build.build_number
    )

    totals = {}
    totals['pass'] = result.pass_count
    totals['error'] = result.error_count
    totals['fail'] = result.fail_count
    totals['count'] = result.total_count
    totals['rate'] = decimal_to_percent(result.pass_rate)

    table_data = []
    for test in tests:
        test_data = dict(
            release=release,
            name=name,
            arch=arch,
            variant=variant,
            build_number=build_number,
            testcase=test.testcase,
            url_name=os.path.basename(test.testcase),
            testsuite=os.path.basename(test.testsuite),
            command_type=test.command_type,
            returncode=test.returncode,
            command=test.command,
            image_id=image.id,
        )
        test_data['id'] = test.pk
        table_data.append(test_data)

    table = SmokeTestResultTable(table_data)
    RequestConfig(request, paginate=None).configure(table)

    data = {
        'name': name,
        'image': image,
        'build_number': image.build_number,
        'release': release,
        'tests': tests,
        'totals': totals,
        'jenkins_url': jenkins_url,
        'table': table,
        'artifacts': artifacts,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_test_results,
            release=release,
            variant=variant,
            arch=arch,
            image_id=image_id,
            name=name,
            build_number=image.build_number,
        ),
    }
    return render_to_response(
        'smokeng/test_results.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Testcase details",
    parent=image_test_results,
    needs=[
        'release',
        'variant',
        'arch',
        'build_number',
        'image_id',
        'name',
        'id',
    ],
)
@require_GET
def test_result_detail(
    request,
    release,
    variant,
    arch,
    build_number,
    image_id,
    name,
    id,
):
    test = get_object_or_404(
        SmokeTestResult,
        pk=id,
    )
    artifacts = Artifact.objects.filter(
        jenkins_build=test.result.jenkins_build,
    ).order_by('name')

    results = []
    test_data = dict(
        command_type=test.command_type,
        testsuite=test.testsuite,
        testcase=test.testcase,
        command=test.command,
        returncode=test.returncode,
        stderr=test.stderr,
        stdout=test.stdout,
    )
    results.append(test_data)

    jenkins_url = (
        test.result.jenkins_build.job.url +
        test.result.jenkins_build.build_number
    )

    data = {
        'results': results,
        'name': name,
        'image': test.result.image,
        'build_number': test.result.image.build_number,
        'release': test.result.image.release,
        'jenkins_url': jenkins_url,
        'testcase': test.testcase,
        'artifacts': artifacts,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            test_result_detail,
            release=release,
            variant=variant,
            arch=arch,
            image_id=image_id,
            name=name,
            id=id,
            build_number=test.result.image.build_number,
        ),
    }

    return render_to_response(
        'smokeng/test_result_detail.html',
        data,
        RequestContext(request),
    )
